/*
 * may not be reproduced, published, advertised in any form
 * without the written agreement of the ADM Crew.
 *

 ADMcisco - cisco scanner by plaguez (02/99)
 feed ip to stdin, get cisco ip from stdout and messages from stderr.

 arg is network interface to be used to packets delivery/capture.

 ( so you could do for example: 
 ./zone .com | ./ADMcisco eth0 | ./cisconuke )


 thanks to gov-boi for beta-testing    

 */

/* i hate comments */
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <memory.h>
#include <stdlib.h>
#include <sys/types.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdarg.h>
#include <unistd.h>
#include <syslog.h>
#include <libnet.h>

#ifdef LINUX
#include <netinet/in.h>
#include "linux-include/netinet/in_systm.h"
#include "linux-include/netinet/ip.h"
#include "linux-include/netinet/tcp.h"
#else
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#endif

#include <pcap.h>
#define APPNAME "ADMdialouts"
#include <debug.h>

#define CISCO "cisco"
#define MAXSIZE 1024
#define CISCO_PORT 1999
#define PROMISC 0


int ETHHDRSIZE = 0;
extern int errno;
struct pcap *pcap_d;
int verbose = 0;
FILE *outfile=NULL;


void collect(u_char * u, struct pcap_pkthdr *p, u_char * packet)
{
    struct ip *ip = (struct ip *) (packet + ETHHDRSIZE);
    u_char *align_buf = NULL;
    int i, len = 1024 * (p->len / 1024 + 1);

    if (verbose)
	DBG("got packet!@");
    align_buf = (u_char *) malloc(len);
    memset(align_buf, '\0', len);
    memcpy(align_buf, (char *) ip, p->len);
    ip = (struct ip *) align_buf;
    for (i = 0; i < (p->len - ETHHDRSIZE); i++)
	if (!align_buf[i])
	    align_buf[i] = '\xff';
    if (strstr(align_buf, CISCO)) {
	if (verbose)
	    fprintf(stderr, "\nFound cisco at: %s\n", host_lookup(ip->ip_src.s_addr, 1));
	fprintf(outfile, "%s\n", host_lookup(ip->ip_src.s_addr, 1));
	fflush(outfile);
    } else if (verbose)
	DBG("nothing.");

    free(align_buf);
    return;
}

void child_handler(int sig)
{
    exit(1);
}

void server(char *device)
{
    char buf[MAXSIZE], errbuf[MAXSIZE], *packet;
    u_long dst_ip = 0, my_ip = 0;
    int sockfd, c;
    struct link_int *l;
    
    fclose(outfile);

    signal(SIGCHLD, child_handler);
    l = libnet_open_link_interface(device, errbuf);
    my_ip = libnet_get_ipaddr(l, device, errbuf);
    if ((sockfd = open_raw_sock(IPPROTO_RAW)) == -1)
	FATAL("open_raw_sock: %s", strerror(errno));
    packet = malloc(TCP_H + IP_H);
    if (!packet)
	FATAL("packet malloc: %s", strerror(errno));
    memset(packet, '\0', TCP_H + IP_H);
    memset(buf, '\0', MAXSIZE);
    seed_prand();
    if (verbose)
	fprintf(stderr, "\nReading ip from stdin...\n");
    while (fgets(buf, MAXSIZE - 1, stdin)) {
	if (!(dst_ip = name_resolve(buf, 1)))
	    ERROR("wrong ip or can't resolve %s", buf);
	else {
	    build_ip(TCP_H, 0, get_prand(PRu16), 0, get_prand(PR8), IPPROTO_TCP, htonl(my_ip),
		     dst_ip, NULL, 0, packet);
	    build_tcp(get_prand(PRu16), CISCO_PORT, get_prand(PRu32), get_prand(PRu32),
		    TH_SYN, get_prand(PRu16), 0, NULL, 0, packet + IP_H);
	    do_checksum(packet, IPPROTO_TCP, TCP_H);
	    c = write_ip(sockfd, packet, TCP_H + IP_H);
	    if (c < TCP_H + IP_H)
		ERROR("write_ip error! (need: %i wrote: %i)", TCP_H + IP_H, c);
	}
    }
    free(packet);
    return;
}

int main(int argc, char **argv)
{
    char ebuf[255], errbuf[PCAP_ERRBUF_SIZE], pcap_filter[MAXSIZE],o;
    struct bpf_program prog;
    u_long network, netmask;

    if (argc < 2)
	FATAL("\nusage: %s [-o outfile] [-v] interface", argv[0]);

    outfile=stdout;

    while ((o = getopt(argc, argv, "o:v")) != -1)
	switch (o) {
	case 'o':
	    if (optarg)
		outfile = fopen(optarg,"a");
	    if(!outfile)
	    {
		    perror("fopen");
		    exit(1);
	    }
	    break;
	case 'v':
	    verbose++;
	    break;
	default:
	    fprintf(stderr,"\nUnknown option, -%c.",o);
	    exit(1); 
	    break;
	}

    argc -= optind;
    argv += optind;

    setpgrp();
    if (fork())
	server(*argv);

    sprintf(pcap_filter, "src port 1999");
    if (pcap_lookupnet(*argv, (bpf_u_int32 *) & network, (bpf_u_int32 *) & netmask, errbuf) < 0)
	FATAL("pcap_lookupnet: %s", errbuf);

    if (!(pcap_d = pcap_open_live(*argv, 8024, PROMISC, 1000, ebuf)))
	FATAL("pcap_open_live: %s", ebuf);


    if (pcap_compile(pcap_d, &prog, pcap_filter, 1, netmask) < 0)
	FATAL("pcap_compile: %s", errbuf);

    if (pcap_setfilter(pcap_d, &prog) < 0)
	FATAL("pcap_setfilter: %s", errbuf);

    switch (pcap_datalink(pcap_d)) {
    case DLT_NULL:
	ETHHDRSIZE = 4;
	break;
    case DLT_EN10MB:
    case DLT_EN3MB:
	ETHHDRSIZE = 14;
	break;
    case DLT_PPP:
	ETHHDRSIZE = 4;
	break;
    case DLT_SLIP:
	ETHHDRSIZE = 16;
	break;
    case DLT_FDDI:
	ETHHDRSIZE = 21;
	break;
    case DLT_RAW:
	ETHHDRSIZE = 0;
	break;
    default:
	FATAL("init_pcap : Unknown device type!");
	break;
    }

    while (1)
	pcap_loop(pcap_d, -1, (pcap_handler) collect, NULL);
    return (0);
}
